/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.notifications.preferences.internal;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.xwiki.component.annotation.Component;
import org.xwiki.component.phase.Initializable;
import org.xwiki.component.phase.InitializationException;
import org.xwiki.model.internal.reference.EntityReferenceFactory;
import org.xwiki.model.reference.DocumentReference;
import org.xwiki.model.reference.EntityReference;
import org.xwiki.model.reference.WikiReference;
import org.xwiki.notifications.NotificationException;
import org.xwiki.notifications.preferences.NotificationPreference;
import org.xwiki.notifications.preferences.internal.cache.UnboundedEntityCacheManager;

/**
 * Wrap the default {@link ModelBridge} to store in the execution context the notification preferences to avoid fetching
 * them several time during the same HTTP request.
 *
 * @version $Id: 91c9370f46233aa6bb92e6cf04ef5549409bd90a $
 * @since 9.5RC1
 */
@Component
@Named("cached")
@Singleton
public class CachedModelBridge implements ModelBridge, Initializable
{
    private static final String CACHE_NAME = "NotificationsPreferences";

    @Inject
    private ModelBridge modelBridge;

    @Inject
    private UnboundedEntityCacheManager cacheManager;

    @Inject
    private EntityReferenceFactory referenceFactory;

    private Map<EntityReference, List<NotificationPreference>> preferenceCache;

    @Override
    public void initialize() throws InitializationException
    {
        // TODO: optimize the invalidation not not be on every user document modification
        this.preferenceCache = this.cacheManager.createCache(CACHE_NAME, true);
    }

    @Override
    public List<NotificationPreference> getNotificationsPreferences(DocumentReference userReference)
        throws NotificationException
    {
        if (userReference == null) {
            return Collections.emptyList();
        }

        List<NotificationPreference> preferences = this.preferenceCache.get(userReference);
        if (preferences != null) {
            return preferences;
        }

        preferences = this.modelBridge.getNotificationsPreferences(userReference);

        this.preferenceCache.put(this.referenceFactory.getReference(userReference), preferences);

        return preferences;
    }

    @Override
    public List<NotificationPreference> getNotificationsPreferences(WikiReference wikiReference)
        throws NotificationException
    {
        if (wikiReference == null) {
            return Collections.emptyList();
        }

        List<NotificationPreference> preferences = this.preferenceCache.get(wikiReference);
        if (preferences != null) {
            return preferences;
        }

        preferences = this.modelBridge.getNotificationsPreferences(wikiReference);

        this.preferenceCache.put(this.referenceFactory.getReference(wikiReference), preferences);

        return preferences;
    }

    @Override
    public void setStartDateForUser(DocumentReference userReference, Date startDate) throws NotificationException
    {
        this.modelBridge.setStartDateForUser(userReference, startDate);
    }

    @Override
    public void saveNotificationsPreferences(DocumentReference userReference,
        List<NotificationPreference> notificationPreferences) throws NotificationException
    {
        this.modelBridge.saveNotificationsPreferences(userReference, notificationPreferences);
    }

    /**
     * @param reference the reference of the entity to invalidate
     * @since 13.4.4
     * @since 12.10.10
     * @since 13.8RC1
     */
    public void invalidatePreference(EntityReference reference)
    {
        this.preferenceCache.remove(reference);
    }
}
